AC_DEFUN(GPC_PATH_GCCSRC, [
#
# Ok, lets find the gcc source tree
# Warning: transition of version 9 to 10 will break this algorithm
# because 10 sorts before 9.
# the alternative search directory is involked by --with-gccsrc
#
no_gccsrc=true
AC_MSG_CHECKING(for gcc sources)
AC_ARG_WITH(gccsrc, [  --with-gccsrc           directory where gcc sources are], with_gccsrc=${withval})
AC_CACHE_VAL(ac_cv_gccsrc,[
# first check to see if --with-gccsrc was specified
if test x"${with_gccsrc}" != x ; then
  if test -f ${with_gccsrc}/c-parse.in ; then
    ac_cv_gccsrc=${with_gccsrc}
  else
    AC_MSG_ERROR([${with_gccsrc} directory doesn't contain gcc sources])
  fi
fi
# next check relative to current directory.
#
# since ls returns lowest version numbers first, reverse its output
if test x"${ac_cv_gccsrc}" = x ; then
  for i in \
		../gcc \
		`ls -dr ../gcc-[[0-9]]* 2>/dev/null` \
		../../gcc \
		`ls -dr ../../gcc-[[0-9]]* 2>/dev/null` \
		../../../gcc \
		`ls -dr ../../../gcc-[[0-9]]* 2>/dev/null` ; do
    if test -f $i/c-parse.in ; then
      ac_cv_gccsrc=$i
      break
    fi
  done
fi
# next check relative to private source directory
#
# since ls returns lowest version numbers first, reverse its output
if test x"${ac_cv_gccsrc}" = x ; then
  for i in \
		${srcdir}/../gcc \
		`ls -dr ${srcdir}/../gcc-[[0-9]]* 2>/dev/null` \
		${srcdir}/../../gcc \
		`ls -dr ${srcdir}/../../gcc-[[0-9]]* 2>/dev/null` \
		${srcdir}/../../../gcc \
		`ls -dr ${srcdir}/../../../gcc-[[0-9]]* 2>/dev/null` ; do
    if test -f $i/c-parse.in ; then
      ac_cv_gccsrc=$i
      break
    fi
  done
fi
# finally check in a few common install locations
#
# since ls returns lowest version numbers first, reverse its output
if test x"${ac_cv_gccsrc}" = x ; then
  for i in \
		`ls -dr ${prefix}/src/gcc-[[0-9]]* 2>/dev/null` \
		${prefix}/src/gcc ; do
    if test -f $i/c-parse.in ; then
      ac_cv_gccsrc=`(cd $i; pwd)`
      break
    fi
  done
fi
])
if test x"${ac_cv_gccsrc}" = x ; then
  gccsrc="# no gcc sources found"
  AC_MSG_ERROR([Can't find gcc sources])
fi
if test x"${ac_cv_gccsrc}" != x ; then
  no_gccsrc=""
  gccsrc="${ac_cv_gccsrc}"
  AC_MSG_RESULT([found in ${ac_cv_gccsrc}])
fi
# RTS subdirectory. If ${gccsrc} is a path relative to . then prepend
# ../ to it to form ${rts_gccsrc}
if test `echo ${ac_cv_gccsrc} | cut -c1` = "/" ; then
  rts_gccsrc="${ac_cv_gccsrc}"
else
  rts_gccsrc="../${ac_cv_gccsrc}"
fi
AC_SUBST(gccsrc)
AC_SUBST(rts_gccsrc)
])

#--------------------------------------------------------------------

AC_DEFUN(GPC_PATH_GCCBIN, [
#
# Repeat this exercise for GCC object code
# the alternative search directory is involked by --with-gccbin
#
no_gccbin=true
AC_MSG_CHECKING(for gcc objects)
AC_ARG_WITH(gccbin, [  --with-gccbin           directory where gcc objects are], with_gccbin=${withval})
AC_CACHE_VAL(ac_cv_gccbin,[
# first check to see if --with-gccbin was specified
if test x"${with_gccbin}" != x ; then
  if test -f ${with_gccbin}/xgcc ; then
    ac_cv_gccbin=${with_gccbin}
  else
    AC_MSG_ERROR([${with_gccbin} directory doesn't contain gcc object code])
  fi
fi
# next check relative to current directory.
#
# since ls returns lowest version numbers first, reverse its output
if test x"${ac_cv_gccbin}" = x ; then
  for i in \
		../gcc \
		`ls -dr ../gcc-[[0-9]]* 2>/dev/null` \
		`ls -dr ../gcc-* 2>/dev/null` \
		../../gcc \
		`ls -dr ../../gcc-[[0-9]]* 2>/dev/null` \
		`ls -dr ../../gcc-* 2>/dev/null` \
		../../../gcc \
		`ls -dr ../../../gcc-[[0-9]]* 2>/dev/null` \
		`ls -dr ../../gcc-* 2>/dev/null` ; do
    if test -f $i/xgcc ; then
      ac_cv_gccbin=$i
      break
    fi
  done
fi
# next check relative to private source directory
#
# since ls returns lowest version numbers first, reverse its output
if test x"${ac_cv_gccbin}" = x ; then
  for i in \
		${srcdir}/../gcc \
		`ls -dr ${srcdir}/../gcc-[[0-9]]* 2>/dev/null` \
		`ls -dr ${srcdir}/../gcc-* 2>/dev/null` \
		${srcdir}/../../gcc \
		`ls -dr ${srcdir}/../../gcc-[[0-9]]* 2>/dev/null` \
		`ls -dr ${srcdir}/../../gcc-* 2>/dev/null` \
		${srcdir}/../../../gcc \
		`ls -dr ${srcdir}/../../../gcc-[[0-9]]* 2>/dev/null` \
		`ls -dr ${srcdir}/../../gcc-* 2>/dev/null` ; do
    if test -f $i/xgcc ; then
      ac_cv_gccbin=$i
      break
    fi
  done
fi
# finally check in a few common install locations
#
# since ls returns lowest version numbers first, reverse its output
if test x"${ac_cv_gccbin}" = x ; then
  for i in \
		`ls -dr ${prefix}/src/gcc-[[0-9]]* 2>/dev/null` \
		${prefix}/src/gcc ; do
    if test -f $i/xgcc ; then
      ac_cv_gccbin=`(cd $i; pwd)`
      break
    fi
  done
fi
])
if test x"${ac_cv_gccbin}" = x ; then
  gccbin="# no gcc sources found"
  AC_MSG_ERROR([Can't find gcc objects])
fi
if test x"${ac_cv_gccbin}" != x ; then
  no_gccbin=""
  gccbin="${ac_cv_gccbin}"
  AC_MSG_RESULT([found in ${ac_cv_gccbin}])
fi
# RTS subdirectory. If ${gccbin} is a path relative to . then prepend
# ../ to it to form ${rts_gccbin}
if test `echo ${ac_cv_gccbin} | cut -c1` = "/" ; then
  rts_gccbin="${ac_cv_gccbin}"
else
  rts_gccbin="../${ac_cv_gccbin}"
fi
AC_SUBST(gccbin)
AC_SUBST(rts_gccbin)
])

#--------------------------------------------------------------------

AC_DEFUN(GPC_GCC_VERSION, [
AC_MSG_CHECKING(for gcc version)
AC_CACHE_VAL(ac_cv_gcc_version,[
ac_cv_gcc_version=`grep '^version=' ${gccbin}/Makefile | sed -e 's/version=//'`])
if test x"${ac_cv_gcc_version}" = x ; then
  gcc_version="# no gcc version found"
  AC_MSG_ERROR([Can't determine gcc version])
else
  gcc_version="${ac_cv_gcc_version}"
  req_version=`grep '^GCCVERSION=' ${srcdir}/Makefile.in | sed -e 's/GCCVERSION=//'`
  if test "${gcc_version}" != ${req_version} ; then
    AC_MSG_ERROR([found gcc ${gcc_version}, require gcc ${req_version}])
  else
    AC_MSG_RESULT([${gcc_version}])
  fi
fi
AC_SUBST(gcc_version)
])

#--------------------------------------------------------------------

dnl Extract target from GCC Makefile
AC_DEFUN(GPC_GCC_TARGET, [
AC_MSG_CHECKING(for gcc target system type)
AC_CACHE_VAL(ac_cv_gcc_target,[
# Cannot disable autoconf's --target switch, but halt if we find one.
if test x"${target}" != xNONE ; then
  AC_MSG_ERROR([do NOT specify --target when you configure gpc, gpc uses the target you selected when you configured gcc])
fi
])
if test -f x${gccbin}/Makefile ; then
  AC_MSG_ERROR([${gccbin}/Makefile not found])
fi
ac_cv_gcc_target=`grep '^target=' ${gccbin}/Makefile | sed -e 's/target=//'`
if test x"${ac_cv_gcc_target}" = x ; then
  gcc_target="# no gcc target found"
  AC_MSG_ERROR([Can't find gcc target])
else
  gcc_target="${ac_cv_gcc_target}"
  AC_MSG_RESULT([${ac_cv_gcc_target}])
fi
target="${gcc_target}"
AC_SUBST(gcc_target)
])

#--------------------------------------------------------------------

AC_DEFUN(GPC_GCC_CROSS, [
AC_MSG_CHECKING(whether building a crosscompiler)
AC_CACHE_VAL(ac_cv_gcc_cross,[
ac_cv_gcc_cross=`grep '^CROSS=' ${gccbin}/Makefile | sed -e 's/CROSS=//'`])
if test x"${ac_cv_gcc_cross}" = x ; then
  cross_compiling="no"
  ac_cv_c_cross="no"
  cross_gcc=""
  AC_MSG_RESULT([no])
else
  cross_compiling="yes"
  ac_cv_c_cross="yes"
  cross_gcc="${ac_cv_gcc_cross}"
  AC_MSG_RESULT([yes])
fi
AC_SUBST(cross_gcc)
])

#--------------------------------------------------------------------

AC_DEFUN(GPC_PROG_AR, [
#
# Seek archiver utility.
#
no_ar=true
AC_MSG_CHECKING(for ar)
AC_ARG_WITH(ar, [  --with-ar               ar utility to use], with_ar=${withval})
AC_CACHE_VAL(ac_cv_prog_ar,[
# first check to see if --with-ar was specified
if test x"${with_ar}" != x ; then
  ac_cv_prog_ar=${with_ar}
fi
if test x"${ac_cv_prog_ar}" = x ; then
  # See if we're crosscompiling.
  #
  #
  if test "${cross_compiling}" = "yes" ; then
    if test -f ${prefix}/bin/${target}-ar; then
      ac_cv_prog_ar=${target}-ar 
    else
      if test -f ${prefix}/${target}/bin/ar; then
        ac_cv_prog_ar=${prefix}/${target}/bin/ar
      fi
    fi
  else
    ac_cv_prog_ar=ar
  fi
fi
])
if test x"${ac_cv_prog_ar}" = x ; then
  AR="# no ar found"
  AC_MSG_ERROR([Unable to find cross-ar. Rerun configure with --with-ar switch])
else
  no_ar=""
  AR="${ac_cv_prog_ar}"
  AC_MSG_RESULT([${ac_cv_prog_ar}])
fi
AC_SUBST(AR)
])

#--------------------------------------------------------------------

AC_DEFUN(GPC_PROG_RANLIB, [
#
# Seek archiver utility.
#
no_ranlib=true
AC_MSG_CHECKING(for ranlib)
AC_ARG_WITH(ranlib, [  --with-ranlib           ranlib utility to use], with_ranlib=${withval})
AC_CACHE_VAL(ac_cv_prog_ranlib,[
# first check to see if --with-ranlib was specified
if test x"${with_ranlib}" != x ; then
  ac_cv_prog_ranlib=${with_ranlib}
fi
if test x"${ac_cv_prog_ranlib}" = x ; then
  # See if we're crosscompiling.
  #
  #
  if test "${cross_compiling}" = "yes" ; then
    if test -f ${prefix}/bin/${target}-ranlib; then
      ac_cv_prog_ranlib=${target}-ranlib
    else
      if test -f ${prefix}/${target}/bin/ranlib; then
        ac_cv_prog_ranlib=${prefix}/${target}/bin/ranlib
      fi
    fi
  else
    ac_cv_prog_ranlib=ranlib
  fi
fi
])
if test x"${ac_cv_prog_ranlib}" = x ; then
  RANLIB="# no ranlib found"
  AC_MSG_ERROR([Unable to find cross-ranlib. Rerun configure with --with-ranlib switch])
else
  no_ranlib=""
  RANLIB="${ac_cv_prog_ranlib}"
  AC_MSG_RESULT([${ac_cv_prog_ranlib}])
fi
AC_SUBST(RANLIB)
])

#--------------------------------------------------------------------

AC_DEFUN(GPC_PROG_RTS_GCC, [
#
# RTS must be compiled with GCC.
# The GCC upon which GPC is based could be used.
# If you (like me) have a bug-fixed gcc installed, but the bugfixes
# break gpc (sources) then you know why I search for installed gcc first,
# then for ${gccbin}/xgcc -B{gccbin}/
#
AC_MSG_CHECKING(for gcc to use for RTS)
AC_ARG_WITH(rtsgcc, [  --with-rtsgcc           gcc to use for RTS], with_rtsgcc=${withval})
AC_CACHE_VAL(ac_cv_prog_rts_gcc,[
# first check to see if --with-rtsgcc was specified
if test x"${with_rtsgcc}" != x ; then
  ac_cv_prog_rts_gcc=${with_rtsgcc}
fi
if test x"${ac_cv_prog_rts_gcc}" = x ; then
  # See if we're crosscompiling.
  if test "${ac_cv_c_cross}" = "yes" ; then
    if test -f ${prefix}/bin/${target}-gcc; then
      ac_cv_prog_rts_gcc=${target}-gcc
    else
      if test -f ${prefix}/${target}/bin/gcc; then
        ac_cv_prog_ranlib=${prefix}/${target}/bin/gcc
      else
        ac_cv_prog_rts_gcc="${gccbin}/xgcc -B${gccbin}/"
      fi
    fi
  else
    if test "${ac_cv_prog_gcc}" = "yes" ; then
      ac_cv_prog_rts_gcc="gcc"
    else
      ac_cv_prog_rts_gcc="${gccbin}/xgcc -B${gccbin}/"
    fi
  fi
fi
])
if test x"${ac_cv_prog_rts_gcc}" != x ; then
  rtsgcc="${ac_cv_prog_rts_gcc}"
  AC_MSG_RESULT([${ac_cv_prog_rts_gcc}])
fi
RTS_GCC="${rtsgcc}"
AC_SUBST(RTS_GCC)
])

#--------------------------------------------------------------------

AC_DEFUN(GPC_GCC_PREFIX, [
AC_MSG_CHECKING(for gcc prefix)
AC_CACHE_VAL(ac_cv_gcc_prefix,[
ac_cv_gcc_prefix=`grep '^prefix =' ${gccbin}/Makefile | sed -e 's/prefix = //'`])
if test x"${ac_cv_gcc_prefix}" = x ; then
  gcc_prefix="# no gcc prefix found"
  AC_MSG_ERROR([Can't determine gcc prefix])
else
  gcc_prefix="${ac_cv_gcc_prefix}"
  AC_MSG_RESULT([${ac_cv_gcc_prefix}])
fi
prefix="${gcc_prefix}"
AC_SUBST(gcc_prefix)
])

#--------------------------------------------------------------------

AC_DEFUN(GPC_GCC_LOCAL_PREFIX, [
AC_MSG_CHECKING(for gcc local_prefix)
AC_CACHE_VAL(ac_cv_gcc_local_prefix,[
ac_cv_gcc_local_prefix=`grep '^local_prefix =' ${gccbin}/Makefile | sed -e 's/local_prefix = //'`])
if test x"${ac_cv_gcc_local_prefix}" = x ; then
  gcc_local_prefix="# no gcc local_prefix found"
  AC_MSG_ERROR([Can't determine gcc local_prefix])
else
  gcc_local_prefix="${ac_cv_gcc_local_prefix}"
  AC_MSG_RESULT([${ac_cv_gcc_local_prefix}])
fi
local_prefix="${gcc_local_prefix}"
AC_SUBST(gcc_local_prefix)
])

#--------------------------------------------------------------------

AC_DEFUN(GPC_GCC_EXTRA_OBJS, [
AC_MSG_CHECKING(for gcc extra_objs)
AC_CACHE_VAL(ac_cv_gcc_extra_objs,[
ac_cv_gcc_extra_objs=`grep '^EXTRA_OBJS = ' ${gccbin}/Makefile | sed -e 's/EXTRA_OBJS = //'`])
if test x"${ac_cv_gcc_extra_objs}" = x ; then
  gcc_extra_objs=""
  AC_MSG_RESULT([none])
else
  gcc_extra_objs="${gccbin}/${ac_cv_gcc_extra_objs}"
  AC_MSG_RESULT([${ac_cv_gcc_extra_objs}])
fi
AC_SUBST(gcc_extra_objs)
])

#--------------------------------------------------------------------

AC_DEFUN(GPC_GCC_OUT_OBJECT_FILE, [
AC_MSG_CHECKING(for gcc out_object_file)
AC_CACHE_VAL(ac_cv_gcc_out_object_file,[
ac_cv_gcc_out_object_file=`grep '^out_object_file=' ${gccbin}/Makefile | sed -e 's/out_object_file=//'`])
if test x"${ac_cv_gcc_out_object_file}" = x ; then
  out_object_file="# no gcc out_object_file found"
  AC_MSG_ERROR([Can't determine gcc out_object_file])
else
  out_object_file="${ac_cv_gcc_out_object_file}"
  AC_MSG_RESULT([${ac_cv_gcc_out_object_file}])
fi
AC_SUBST(out_object_file)
])
